﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TechNote56_SmartIO_RS232_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // FrameStyle
        private string[] m_strFrameStyle = { "STXANDETX(1-1Byte)", "STXANDETX(2-1Byte)", "STXANDETX(1-2Byte)", "STXANDETX(2-2Byte)", "STXANDETX_RTO(1-1Byte)", "STXANDETX_RTO(2-1Byte)", "STXANDETX_RTO(1-2Byte)",
                                           "STXANDETX_RTO(2-2Byte)","STXONLY(1Byte)", "STXONLY_RTO(1Byte)", "STXONLY(2Byte)", "STXONLY_RTO(2Byte)", "ETXONLY(1Byte)", "ETXONLY_RTO(1Byte)", "ETXONLY(2Byte)",
                                           "ETXONLY_RTO(2Byte)", "NONEFRAME_READTIMEOUT"};
        // BaudRate
        private string[] m_strBaudRate = { "110", "300", "600", "1200", "2400", "4800", "9600", "14400", "19200", "38400", "56000", "57600", "115200" };

        private void Form1_Load(object sender, EventArgs e)
        {
            // 현재 Form을 MainForm으로 설정합니다.
            smartForm1.MainForm = this;

            // FrameSeparationType을 설정하는 SmartComboBox에 아이템을 추가
            cbFrameStyle.DataSource = m_strFrameStyle;
            cbFrameStyle.SelectItemIndex = 0;

            // BaudRate를 설정하는 SmartComboBox에 아이템을 추가
            cbBaudRate.DataSource = m_strBaudRate;
            cbBaudRate.SelectItemIndex = 6;
        }

        // 송/수신 데이터의 구조가 Frame과 NoneFrame에 따라 FrameSeparationType을 구분하여 설정
        private void btnOpen_Click(object sender, EventArgs e)
        {
            // 선택한 인덱스를 저장
            int iFrameSelect = cbFrameStyle.SelectItemIndex;

            // smartSerialPort가 Open되지 않은 경우
            if (smartSerialPort1.IsOpen == false)
            {
                // STX, ETX를 저장할 바이트 배열
                byte[] bSTX = new byte[2];
                byte[] bETX = new byte[2];

                // SmartSerialPort STX, ETX 초기화
                smartSerialPort1.STXCodes = null;
                smartSerialPort1.ETXCodes = null;

                // STX, ETX를 변환하여 배열에 저장
                bSTX[0] = Convert.ToByte(labSTX1.Text);
                bSTX[1] = Convert.ToByte(labSTX2.Text);
                bETX[0] = Convert.ToByte(labETX1.Text);
                bETX[1] = Convert.ToByte(labETX2.Text);

                switch (iFrameSelect)
                {
                    case 0:
                    case 4:
                        // STX(1Byte), ETX(1Byte)를 사용하는 경우
                        smartSerialPort1.STXCode = bSTX[0];
                        smartSerialPort1.ETXCode = bETX[0];
                        break;
                    case 1:
                    case 5:
                        // STX(2Byte), ETX(1Byte)를 사용하는 경우
                        smartSerialPort1.STXCodes = bSTX;
                        smartSerialPort1.ETXCode = bETX[0];
                        break;
                    case 2:
                    case 6:
                        // STX(1Byte), ETX(2Byte)를 사용하는 경우
                        smartSerialPort1.STXCode = bSTX[0];
                        smartSerialPort1.ETXCodes = bETX;
                        break;
                    case 3:
                    case 7:
                        // STX(2Byte), ETX(2Byte)를 사용하는 경우
                        smartSerialPort1.STXCodes = bSTX;
                        smartSerialPort1.ETXCodes = bETX;
                        break;
                    case 8:
                    case 9:
                        // STX(1Byte)를 사용하는 경우
                        smartSerialPort1.STXCode = bSTX[0];
                        break;
                    case 10:
                    case 11:
                        // STX(2Byte)를 사용하는 경우
                        smartSerialPort1.STXCodes = bSTX;
                        break;
                    case 12:
                    case 13:
                        // ETX(1Byte)를 사용하는 경우
                        smartSerialPort1.ETXCode = bETX[0];
                        break;
                    case 14:
                    case 15:
                        // ETX(2Byte)를 사용하는 경우
                        smartSerialPort1.ETXCodes = bETX;
                        break;
                    default:
                        break;
                }
                
                // SmartComboBox로 선택한 Item에 따라 Baud_Rate 설정
                smartSerialPort1.Baud_Rate = (SmartX.SmartSerialPort.BAUDRATE)(Convert.ToInt32(cbBaudRate.Text));

                // Port번호 선택. RS232블럭을 사용하는 경우 IEC667/1000-Series은 COM4 포트만 사용 가능합니다.
                if (radCOM1.Checked == true)
                {
                    smartSerialPort1.PortNo = SmartX.SmartSerialPort.COMPORTNO.COM1;
                }
                else if (radCOM2.Checked == true)
                {
                    smartSerialPort1.PortNo = SmartX.SmartSerialPort.COMPORTNO.COM2;
                }
                else if (radCOM3.Checked == true)
                {
                    smartSerialPort1.PortNo = SmartX.SmartSerialPort.COMPORTNO.COM3; 
                }
                else if (radCOM4.Checked == true)
                {
                    smartSerialPort1.PortNo = SmartX.SmartSerialPort.COMPORTNO.COM4;
                }
                else
                {
                    smartSerialPort1.PortNo = SmartX.SmartSerialPort.COMPORTNO.COM7;
                }

                if (radCOM4.Checked == false)
                {
                    SmartX.SmartMessageBox.Show("RS232 Block을 사용하는 경우 IEC667/1000은 COM4 포트만 사용할 수 있습니다.");
                }

                // Error Check Mode
                if (radNone.Checked == true)
                {
                    smartSerialPort1.ErrorCheckMode = SmartX.SmartSerialPort.ERRORCHECK.NONE;
                }
                else if (radCheckSum8.Checked == true)
                {
                    smartSerialPort1.ErrorCheckMode = SmartX.SmartSerialPort.ERRORCHECK.CHECKSUM8;
                }
                else if (radCheckSum16.Checked == true)
                {
                    smartSerialPort1.ErrorCheckMode = SmartX.SmartSerialPort.ERRORCHECK.CHECKSUM16;
                }
                else if (radCRC16.Checked == true)
                {
                    smartSerialPort1.ErrorCheckMode = SmartX.SmartSerialPort.ERRORCHECK.CRC16;
                }
                else
                {
                    smartSerialPort1.ErrorCheckMode = SmartX.SmartSerialPort.ERRORCHECK.CRC32;
                }

                // ErrorCodeLocation
                if (radHead.Checked == true)
                {
                    smartSerialPort1.ErrorCode_Location = SmartX.SmartSerialPort.ERRORCODELOCATION.HEADER;
                }
                else
                {
                    smartSerialPort1.ErrorCode_Location = SmartX.SmartSerialPort.ERRORCODELOCATION.TAIL;
                }

                if (radEvent.Checked == true)
                {
                    smartSerialPort1.ReceiveDetect = SmartX.SmartSerialPort.RECEIVEDETECTTYPE.EVENT_QUEUE;
                }
                else
                {
                    smartSerialPort1.ReceiveDetect = SmartX.SmartSerialPort.RECEIVEDETECTTYPE.POLLING_NONEQUEUE;
                }

                // ReadTimeout 설정 (ReadTimeout을 사용하는 FrameType의 경우 반드시 설정해야 합니다.)
                smartSerialPort1.ReadTimeout = Convert.ToInt32(udReadTimeOut.Value);
                // Frame의 BufferSize를 설정
                smartSerialPort1.FrameBufferSize = Convert.ToUInt32(udBufferSize.Value);
                // ReceiveFrameDebugMode 설정 (True는 FrameData[STX+Data+ErrorCheckCode+ETX] 송/수신, False는 Data만 송/수신)
                smartSerialPort1.ReceiveFrameDebugMode = chkDebugMode.Checked;
                smartSerialPort1.RS485SoftwareDetection = false;

                // Port가 열려있지 않다면 Port를 Open합니다.
                if (smartSerialPort1.IsOpen == false)
                {
                    smartSerialPort1.Open();
                    labStatus.Text = "SerialPort Opened";
                }
            }
        }

        private void cbFrameStyle_SelectedIndexChanged(object sender, EventArgs e)
        {
            // 선택한 인덱스를 저장
            int iFrameSelect = cbFrameStyle.SelectItemIndex;

            // labSTX1, labETX1, labSTX2, labETX2 라벨을 보이도록 설정
            labSTX1.Visible = true;
            labETX1.Visible = true;
            labSTX2.Visible = true;
            labETX2.Visible = true;

            // STX1을 사용하지 않는 경우 
            // STX Code를 사용하지 않는 경우이며, ETXOnly, NoneFrameReadTimeout에 해당합니다.
            if (IsCompare(iFrameSelect, 12, 13, 14, 15, 16) == true)
            {
                // STX1 라벨을 안보이도록 설정
                labSTX1.Visible = false;
            }
            // STX2를 사용하지 않는 경우
            // STX 2Byte를 사용하지 않는 경우이며, STX 1Byte, ETXOnly, NoneFrameReadTimeout에 해당합니다.
            if (IsCompare(iFrameSelect, 0, 2, 4, 6, 8, 9, 12, 13, 14, 15, 16) == true)
            {
                // STX2 라벨을 안보이도록 설정
                labSTX2.Visible = false;
            }
            // ETX1을 사용하지 않는 경우
            // ETX Code를 사용하지 않는 경우이며, STXOnly, NoneFrameReadTimeout에 해당합니다.
            if (IsCompare(iFrameSelect, 8, 9, 10, 11, 16) == true)
            {
                // ETX1 라벨을 안보이도록 설정
                labETX1.Visible = false;
            }
            // ETX2를 사용하지 않는 경우
            // ETX 2Byte를 사용하지 않는 경우이며, STXOnly, ETX1Byte, NoneFrameReadTimeout에 해당합니다.
            if (IsCompare(iFrameSelect, 0, 1, 4, 5, 8, 9, 10, 11, 12, 13, 16) == true)
            {
                // ETX1 라벨을 안보이도록 설정
                labETX2.Visible = false;
            }

            // 선택한 Frame에 맞춰서 FRAMESEPARATIONTYPES를 설정합니다.
            switch (iFrameSelect)
            {
                case 0:
                case 1:
                case 2:
                case 3:
                    // STXANDETX
                    smartSerialPort1.FrameSeparationType = SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.STXANDETX;
                    break;
                case 4:
                case 5:
                case 6:
                case 7:
                    // STXANDETX_READTIMEOUT
                    smartSerialPort1.FrameSeparationType = SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.STXANDETX_READTIMEOUT;
                    break;
                case 8:
                case 10:
                    // STXONLY
                    smartSerialPort1.FrameSeparationType = SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.STXONLY;
                    break;
                case 9:
                case 11:
                    // STXONLY_READTIMEOUT
                    smartSerialPort1.FrameSeparationType = SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.STXONLY_READTIMEOUT;
                    break;
                case 12:
                case 14:
                    // ETXONLY_READTIMEOUT
                    smartSerialPort1.FrameSeparationType = SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.ETXONLY;
                    break;
                case 13:
                case 15:
                    // STXONLY_READTIMEOUT(2Byte)
                    smartSerialPort1.FrameSeparationType = SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.ETXONLY_READTIMEOUT;
                    break;
                case 16:
                    // NONEFRAME_READTIMEOUT
                    smartSerialPort1.FrameSeparationType = SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.NONEFRAME_READTIMEOUT;
                    break;
                default:
                    break;
            }
        }

        // Data 전송
        private void btnSend_Click(object sender, EventArgs e)
        {
            SendData();

            smartTimer1.Start();
        }

        // Repeat CheckBox가 체크되어 있는 경우 연속 송신
        private void smartTimer1_Tick(object sender, EventArgs e)
        {
            if (chkRepeat.Checked == false)
            {
                smartTimer1.Stop();
                return;
            }

            SendData();
        }

        // iRefValue값과 iArgs값을 비교하는 함수
        private bool IsCompare(int iRefValue, params int[] iArgs)
        {
            // 값을 비교하여 일치하는 값이 있다면 True 없다면 False
            return iArgs.Contains(iRefValue);
        }

        // Data를 Send하기 위한 함수
        private void SendData()
        {
            // Frame Style에 따른 Send Method 분류
            // 1. Frame-Event / 2. Frame-Polling
            if (smartSerialPort1.FrameSeparationType == SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.NONEFRAME_READTIMEOUT)
            {
                if (radSendAscii.Checked == true)
                {
                    smartSerialPort1.WriteNoneFrame(labSendData.Text, SmartX.SmartSerialPort.CODETYPES.ASCIICODE);
                }
                else if (radSendUnicode.Checked == true)
                {
                    smartSerialPort1.WriteNoneFrame(labSendData.Text, SmartX.SmartSerialPort.CODETYPES.UNICODE);
                }
                else
                {
                    byte[] bByteArray = Encoding.UTF8.GetBytes(labSendData.Text);
                    smartSerialPort1.WriteNoneFrame(bByteArray);
                }
            }
            else
            {
                if (radSendAscii.Checked == true)
                {
                    smartSerialPort1.WriteFrame(labSendData.Text, SmartX.SmartSerialPort.CODETYPES.ASCIICODE);
                }
                else if (radSendUnicode.Checked == true)
                {
                    smartSerialPort1.WriteFrame(labSendData.Text, SmartX.SmartSerialPort.CODETYPES.UNICODE);
                }
                else
                {
                    // 문자열 --> Byte[] 변환후 전송
                    byte[] bByteArray = Encoding.UTF8.GetBytes(labSendData.Text);
                    smartSerialPort1.WriteFrame(bByteArray);
                }
            }
        }

        // Event 수신 방식
        private void smartSerialPort1_OnReadQueueEvent()
        {
            string strData = "";
            byte[] readByte;
            SmartX.SmartSerialPort.FRAMEDATAREADSTATUS eStatus;

            // 1. Frmae, 2. NoneFrame
            eStatus = smartSerialPort1.ReadQueue(out readByte);

            // ASCII, UniCode, Binary 방식에 따라 구분
            if (radRecvAscii.Checked == true)
            {
                strData = SmartX.SmartSerialPort.ConvertAsciiByteToString(readByte);
            }
            else if (radRecvUnicode.Checked == true)
            {
                strData = SmartX.SmartSerialPort.ConvertUnicodeByteToString(readByte);
            }
            else
            {
                // 수신된 프레임 데이터를 각각의 바이트를 16진수로 표시 합니다
                for (int i = 0; i < readByte.Length; i++)
                {
                    strData += string.Format(" {0:X2}", readByte[i]);
                }
            }

            // ListBox에 데이터 삽입
            if (eStatus == SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.EMPTY)
            {
                lstRecvData.AddItem("EMPTY");
            }
            else if (eStatus == SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.FAILDATA)
            {
                lstRecvData.AddItem("Fail");
            }
            else
            {
                lstRecvData.AddItem(strData);
            }
        }

        private void btnRecvPolling_Click(object sender, EventArgs e)
        {
            string strData = null;
            byte[] readByte = new byte[10];

            SmartX.SmartSerialPort.FRAMEDATAREADSTATUS eStatus = SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.EMPTY;

            // 1. NoneFrame
            if (smartSerialPort1.FrameSeparationType == SmartX.SmartSerialPort.FRAMESEPARATIONTYPES.NONEFRAME_READTIMEOUT)
            {
                eStatus = smartSerialPort1.ReadNoneFrame(out readByte, 1000);
            }
            // 2. Frame
            else 
            {
                eStatus = smartSerialPort1.ReadFrame(out readByte);
            }
            

            // ListBox에 데이터 삽입
            if (eStatus == SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.EMPTY)
            {
                lstRecvData.AddItem("EMPTY");
            }
            else if (eStatus == SmartX.SmartSerialPort.FRAMEDATAREADSTATUS.FAILDATA)
            {
                lstRecvData.AddItem("Fail");
            }
            else
            {
                // ASCII, UniCode, Binary방식에따라 구분
                if (radRecvAscii.Checked == true)
                {
                    strData = SmartX.SmartSerialPort.ConvertAsciiByteToString(readByte);
                }
                else if (radRecvUnicode.Checked == true)
                {
                    strData = SmartX.SmartSerialPort.ConvertUnicodeByteToString(readByte);
                }
                else
                {
                    // 수신된 프레임 데이터를 각각의 바이트를 16진수로 표시 합니다
                    for (int i = 0; i < readByte.Length; i++)
                    {
                        strData += string.Format(" {0:X2}", readByte[i]);
                    }
                }

                lstRecvData.AddItem(strData);
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            if (smartSerialPort1.IsOpen == true)
            {
                smartSerialPort1.Close();
                labStatus.Text = "SerialPort Closed";
            }
        }

        private void btnListClear_Click(object sender, EventArgs e)
        {
            lstRecvData.ClearAll();
        }

        private void RadioButton_Click(object sender, EventArgs e)
        {
            if (radEvent.Checked == true)
            {
                btnRecvPolling.Visible = false;
            }
            else
            {
                btnRecvPolling.Visible = true;
            }
        }
    }
}